Package: stPipe
Type: Package
Title: Upstream pre-processing for Sequencing-Based Spatial
        Transcriptomics
Version: 1.3.0
Authors@R: c(person("Yang", "Xu", role=c("aut", "cre"), email="xu.ya@wehi.edu.au", comment=c(ORCID="0009-0008-3274-6516")), 
        person("Callum", "Sargeant", role=c("aut"), email="sargeant.ca@wehi.edu.au"), 
        person("Shian", "Su", role=c("aut"), email="su.s@wehi.edu.au"), 
        person("Luyi", "Tian", role=c("aut"), email="tian.l@wehi.edu.au"), 
        person("Yunshun", "Chen", role=c("ctb"), email="yuchen@wehi.edu.au"), 
        person("Matthew", "Ritchie", role=c("ctb", "fnd"), email="mritchie@wehi.edu.au"))
biocViews: ImmunoOncology, Software, Sequencing, RNASeq,
        GeneExpression, SingleCell, Visualization, SequenceMatching,
        Preprocessing, QualityControl, GenomeAnnotation, DataImport,
        Spatial, Transcriptomics, Clustering
Description: This package serves as an upstream pipeline for pre-processing sequencing-based spatial transcriptomics data. Functions includes FASTQ trimming, BAM file reformatting, index building, spatial barcode detection, demultiplexing, gene count matrix generation with UMI deduplication, QC, and revelant visualization. Config is an essential input for most of the functions which aims to improve reproducibility.
Depends: R (>= 4.5.0)
Imports: basilisk, data.table, DropletUtils, dplyr, ggplot2, methods,
        pbmcapply, reticulate, rmarkdown, Rcpp, Rhtslib, Rsubread,
        Rtsne, Seurat, SeuratObject, scPipe, shiny,
        SummarizedExperiment, SingleCellExperiment, SpatialExperiment,
        stats, umap, yaml
LinkingTo: Rcpp, Rhdf5lib, testthat, Rhtslib
SystemRequirements: GNU make
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/mritchielab/stPipe
BugReports: https://github.com/mritchielab/stPipe/issues/new
Suggests: knitr, plotly, BiocStyle, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/stPipe
git_branch: devel
git_last_commit: 287ceed
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: yes
Packaged: 2026-04-30 00:52:47 UTC; biocbuild
Author: Yang Xu [aut, cre] (ORCID: <https://orcid.org/0009-0008-3274-6516>),
  Callum Sargeant [aut],
  Shian Su [aut],
  Luyi Tian [aut],
  Yunshun Chen [ctb],
  Matthew Ritchie [ctb, fnd]
Maintainer: Yang Xu <xu.ya@wehi.edu.au>
