# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @name RunDemultiplex
NULL

RunDemultiplex <- function(read_1_fq_path, read_2_fq_path, h5_mapping_path, output_fq_path, n_reads, coord_bc_start, coord_bc_len, umi_start, umi_len, bin_size = 1L) {
    invisible(.Call(`_stPipe_RunDemultiplex`, read_1_fq_path, read_2_fq_path, h5_mapping_path, output_fq_path, n_reads, coord_bc_start, coord_bc_len, umi_start, umi_len, bin_size))
}

